@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -40,6 +40,14 @@
 #include "utils.h"
 #include "prefs_common.h"
 
+#undef MIME_DEBUG
+/* #define MIME_DEBUG */
+#ifdef MIME_DEBUG
+#  define mime_debug_print	debug_print
+#else
+#  define mime_debug_print	1 ? (void)0 : debug_print
+#endif
+
 #define MAX_MIME_LEVEL	64
 
 static GHashTable *procmime_get_mime_type_table	(void);
@@ -245,7 +253,8 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 		return;
 	}
 
-	debug_print("level = %d\n", mimeinfo->level);
+	mime_debug_print("==== enter part\n");
+	mime_debug_print("level = %d\n", mimeinfo->level);
 
 	for (;;) {
 		MimeInfo *partinfo;
@@ -257,7 +266,7 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 		gint b64_pad_len = 0;
 
 		prev_fpos = fpos;
-		debug_print("prev_fpos: %ld\n", fpos);
+		mime_debug_print("prev_fpos: %ld\n", fpos);
 
 		/* scan part header */
 		if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
@@ -266,8 +275,8 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 			mimeinfo->sub = sub = procmime_scan_mime_header(fp);
 			if (!sub) break;
 
-			debug_print("message/rfc822 part (content-type: %s)\n",
-				    sub->content_type);
+			mime_debug_print("message/rfc822 part (content-type: %s)\n",
+					 sub->content_type);
 			sub->level = mimeinfo->level + 1;
 			sub->parent = mimeinfo->parent;
 			sub->main = mimeinfo;
@@ -277,18 +286,25 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 			partinfo = procmime_scan_mime_header(fp);
 			if (!partinfo) break;
 			procmime_mimeinfo_insert(mimeinfo, partinfo);
-			debug_print("content-type: %s\n",
-				    partinfo->content_type);
+			mime_debug_print("content-type: %s\n",
+					 partinfo->content_type);
+			if (partinfo->filename)
+				mime_debug_print("filename: %s\n", partinfo->filename);
+			else if (partinfo->name)
+				mime_debug_print("name: %s\n", partinfo->name);
 		}
 
 		/* begin content */
 		content_pos = ftell(fp);
-		debug_print("content_pos: %ld\n", content_pos);
+		mime_debug_print("content_pos: %ld\n", content_pos);
 
 		if (partinfo->mime_type == MIME_MULTIPART ||
 		    partinfo->mime_type == MIME_MESSAGE_RFC822) {
-			if (partinfo->level < MAX_MIME_LEVEL)
+			if (partinfo->level < MAX_MIME_LEVEL) {
+				mime_debug_print("\n");
+				mime_debug_print("enter to child part:\n");
 				procmime_scan_multipart_message(partinfo, fp);
+			}
 		}
 
 		/* look for next boundary */
@@ -314,10 +330,10 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 			buf[0] = '\0';
 			eom = TRUE;
 		}
-		debug_print("boundary: %s\n", buf);
+		mime_debug_print("boundary: %s\n", buf);
 
 		fpos = ftell(fp);
-		debug_print("fpos: %ld\n", fpos);
+		mime_debug_print("fpos: %ld\n", fpos);
 
 		len = strlen(buf);
 		partinfo->size = fpos - prev_fpos - len;
@@ -326,15 +342,15 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 				b64_content_len / 4 * 3 - b64_pad_len;
 		else
 			partinfo->content_size = fpos - content_pos - len;
-		debug_print("partinfo->size: %d\n", partinfo->size);
-		debug_print("partinfo->content_size: %d\n",
-			    partinfo->content_size);
+		mime_debug_print("partinfo->size: %d\n", partinfo->size);
+		mime_debug_print("partinfo->content_size: %d\n",
+				 partinfo->content_size);
 		if (partinfo->sub && !partinfo->sub->sub &&
 		    !partinfo->sub->children) {
 			partinfo->sub->size =
 				fpos - partinfo->sub->fpos - strlen(buf);
-			debug_print("partinfo->sub->size: %d\n",
-				    partinfo->sub->size);
+			mime_debug_print("partinfo->sub->size: %d\n",
+					 partinfo->sub->size);
 		}
 
 		if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
@@ -347,26 +363,25 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 	}
 
 	g_free(buf);
+	mime_debug_print("==== leave part\n");
 }
 
 void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
 {
-	gchar *buf;
-
-	Xstrdup_a(buf, encoding, return);
+	gchar *enc;
 
 	g_free(mimeinfo->encoding);
+	enc = mimeinfo->encoding = g_strstrip(g_strdup(encoding));
 
-	mimeinfo->encoding = g_strdup(g_strstrip(buf));
-	if (!g_ascii_strncasecmp(buf, "7bit", 4))
+	if (!g_ascii_strncasecmp(enc, "7bit", 4))
 		mimeinfo->encoding_type = ENC_7BIT;
-	else if (!g_ascii_strncasecmp(buf, "8bit", 4))
+	else if (!g_ascii_strncasecmp(enc, "8bit", 4))
 		mimeinfo->encoding_type = ENC_8BIT;
-	else if (!g_ascii_strncasecmp(buf, "quoted-printable", 16))
+	else if (!g_ascii_strncasecmp(enc, "quoted-printable", 16))
 		mimeinfo->encoding_type = ENC_QUOTED_PRINTABLE;
-	else if (!g_ascii_strncasecmp(buf, "base64", 6))
+	else if (!g_ascii_strncasecmp(enc, "base64", 6))
 		mimeinfo->encoding_type = ENC_BASE64;
-	else if (!g_ascii_strncasecmp(buf, "x-uuencode", 10))
+	else if (!g_ascii_strncasecmp(enc, "x-uuencode", 10))
 		mimeinfo->encoding_type = ENC_X_UUENCODE;
 	else
 		mimeinfo->encoding_type = ENC_UNKNOWN;
@@ -1488,7 +1503,7 @@ gchar *procmime_get_mime_type(const gchar *filename)
 	static GHashTable *mime_type_table = NULL;
 	MimeType *mime_type;
 	const gchar *p;
-	gchar *ext;
+	gchar ext[64];
 	static gboolean no_mime_type_table = FALSE;
 
 	if (no_mime_type_table)
@@ -1506,7 +1521,7 @@ gchar *procmime_get_mime_type(const gchar *filename)
 	p = strrchr(filename, '.');
 	if (!p) return NULL;
 
-	Xstrdup_a(ext, p + 1, return NULL);
+	strncpy2(ext, p + 1, sizeof(ext));
 	g_strdown(ext);
 	mime_type = g_hash_table_lookup(mime_type_table, ext);
 	if (mime_type) {
@@ -1680,6 +1695,8 @@ static GList *procmime_parse_mailcap(const gchar *file)
 		list = g_list_append(list, mailcap);
 	}
 
+	fclose(fp);
+
 	return list;
 }
 
