@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #include "utils.h"
 #include "procmsg.h"
@@ -33,6 +34,14 @@
 #include "folder.h"
 #include "codeconv.h"
 
+typedef struct _MsgFlagInfo {
+	guint msgnum;
+	MsgFlags flags;
+} MsgFlagInfo;
+
+static GSList *procmsg_read_cache_queue		(FolderItem	*item,
+						 gboolean	 scan_file);
+
 static void mark_sum_func			(gpointer	 key,
 						 gpointer	 value,
 						 gpointer	 data);
@@ -41,10 +50,8 @@ static GHashTable *procmsg_read_mark_file	(FolderItem	*item);
 static void procmsg_write_mark_file		(FolderItem	*item,
 						 GHashTable	*mark_table);
 
-static FILE *procmsg_open_cache_file_with_buffer(FolderItem	*item,
-						 DataOpenMode	 mode,
-						 gchar		*buf,
-						 size_t		 buf_size);
+static GMappedFile *procmsg_open_cache_file_mmap(FolderItem	*item,
+						 DataOpenMode	 mode);
 
 static gint procmsg_cmp_by_mark			(gconstpointer	 a,
 						 gconstpointer	 b);
@@ -117,75 +124,94 @@ GHashTable *procmsg_to_folder_hash_table_create(GSList *mlist)
 gint procmsg_read_cache_data_str(FILE *fp, gchar **str)
 {
 	gchar buf[BUFFSIZE];
-	gint ret = 0;
+	guint32 len;
+	gchar *tmp = NULL;
+
+	if (fread(&len, sizeof(len), 1, fp) != 1)
+		return -1;
+
+	if (len > G_MAXINT)
+		return -1;
+
+	while (len > 0) {
+		size_t size = MIN(len, BUFFSIZE - 1);
+
+		if (fread(buf, size, 1, fp) != 1) {
+			if (tmp)
+				g_free(tmp);
+			*str = NULL;
+			return -1;
+		}
+
+		buf[size] = '\0';
+		if (tmp) {
+			*str = g_strconcat(tmp, buf, NULL);
+			g_free(tmp);
+			tmp = *str;
+		} else
+			tmp = *str = g_strdup(buf);
+
+		len -= size;
+	}
+
+	return 0;
+}
+
+static gint procmsg_read_cache_data_str_mem(const gchar **p, const gchar *endp, gchar **str)
+{
 	guint32 len;
 
-	if (fread(&len, sizeof(len), 1, fp) == 1) {
-		if (len > G_MAXINT)
-			ret = -1;
-		else {
-			gchar *tmp = NULL;
+	if (endp - *p < sizeof(len))
+		return -1;
 
-			while (len > 0) {
-				size_t size = MIN(len, BUFFSIZE - 1);
+	len = *(const guint32 *)(*p);
+	*p += sizeof(len);
+	if (len > G_MAXINT || len > endp - *p)
+		return -1;
 
-				if (fread(buf, size, 1, fp) != 1) {
-					ret = -1;
-					if (tmp) g_free(tmp);
-					*str = NULL;
-					break;
-				}
+	if (len > 0) {
+		*str = g_strndup(*p, len);
+		*p += len;
+	}
 
-				buf[size] = '\0';
-				if (tmp) {
-					*str = g_strconcat(tmp, buf, NULL);
-					g_free(tmp);
-					tmp = *str;
-				} else
-					tmp = *str = g_strdup(buf);
-
-				len -= size;
-			}
-		}
-	} else
-		ret = -1;
-
-	return ret;
+	return 0;
 }
 
-#define READ_CACHE_DATA(data, fp)				\
+#define READ_CACHE_DATA(data)						\
+{									\
+	if (procmsg_read_cache_data_str_mem(&p, endp, &data) < 0) {	\
+		g_warning("Cache data is corrupted\n");			\
+		procmsg_msginfo_free(msginfo);				\
+		procmsg_msg_list_free(mlist);				\
+		g_mapped_file_free(mapfile);				\
+		return NULL;						\
+	}								\
+}
+
+#define READ_CACHE_DATA_INT(n)					\
 {								\
-	if (procmsg_read_cache_data_str(fp, &data) < 0) {	\
+	if (endp - p < sizeof(guint32)) {			\
 		g_warning("Cache data is corrupted\n");		\
 		procmsg_msginfo_free(msginfo);			\
 		procmsg_msg_list_free(mlist);			\
-		fclose(fp);					\
+		g_mapped_file_free(mapfile);			\
 		return NULL;					\
+	} else {						\
+		n = *(const guint32 *)p;			\
+		p += sizeof(guint32);				\
 	}							\
 }
 
-#define READ_CACHE_DATA_INT(n, fp)				\
-{								\
-	guint32 idata;						\
-								\
-	if (fread(&idata, sizeof(idata), 1, fp) != 1) {		\
-		g_warning("Cache data is corrupted\n");		\
-		procmsg_msginfo_free(msginfo);			\
-		procmsg_msg_list_free(mlist);			\
-		fclose(fp);					\
-		return NULL;					\
-	} else							\
-		n = idata;					\
-}
-
 GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 {
 	GSList *mlist = NULL;
 	GSList *last = NULL;
-	FILE *fp;
+	GMappedFile *mapfile;
+	const gchar *filep;
+	gsize file_len;
+	const gchar *p, *endp;
 	MsgInfo *msginfo;
 	MsgFlags default_flags;
-	gchar file_buf[BUFFSIZE];
 	guint32 num;
 	guint refnum;
 	FolderType type;
@@ -220,37 +246,44 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		g_free(path);
 	}
 
-	if ((fp = procmsg_open_cache_file_with_buffer
-		(item, DATA_READ, file_buf, sizeof(file_buf))) == NULL) {
+	mapfile = procmsg_open_cache_file_mmap(item, DATA_READ);
+	if (!mapfile) {
 		item->cache_dirty = TRUE;
 		return NULL;
 	}
 
-	debug_print("Reading summary cache...");
+	debug_print("Reading summary cache...\n");
 
-	while (fread(&num, sizeof(num), 1, fp) == 1) {
+	filep = g_mapped_file_get_contents(mapfile);
+	file_len = g_mapped_file_get_length(mapfile);
+	endp = filep + file_len;
+	p = filep + sizeof(guint32); /* version */
+
+	while (endp - p >= sizeof(num)) {
 		msginfo = g_new0(MsgInfo, 1);
-		msginfo->msgnum = num;
-		READ_CACHE_DATA_INT(msginfo->size, fp);
-		READ_CACHE_DATA_INT(msginfo->mtime, fp);
-		READ_CACHE_DATA_INT(msginfo->date_t, fp);
-		READ_CACHE_DATA_INT(msginfo->flags.tmp_flags, fp);
 
-		READ_CACHE_DATA(msginfo->fromname, fp);
+		READ_CACHE_DATA_INT(msginfo->msgnum);
 
-		READ_CACHE_DATA(msginfo->date, fp);
-		READ_CACHE_DATA(msginfo->from, fp);
-		READ_CACHE_DATA(msginfo->to, fp);
-		READ_CACHE_DATA(msginfo->newsgroups, fp);
-		READ_CACHE_DATA(msginfo->subject, fp);
-		READ_CACHE_DATA(msginfo->msgid, fp);
-		READ_CACHE_DATA(msginfo->inreplyto, fp);
+		READ_CACHE_DATA_INT(msginfo->size);
+		READ_CACHE_DATA_INT(msginfo->mtime);
+		READ_CACHE_DATA_INT(msginfo->date_t);
+		READ_CACHE_DATA_INT(msginfo->flags.tmp_flags);
 
-		READ_CACHE_DATA_INT(refnum, fp);
+		READ_CACHE_DATA(msginfo->fromname);
+
+		READ_CACHE_DATA(msginfo->date);
+		READ_CACHE_DATA(msginfo->from);
+		READ_CACHE_DATA(msginfo->to);
+		READ_CACHE_DATA(msginfo->newsgroups);
+		READ_CACHE_DATA(msginfo->subject);
+		READ_CACHE_DATA(msginfo->msgid);
+		READ_CACHE_DATA(msginfo->inreplyto);
+
+		READ_CACHE_DATA_INT(refnum);
 		for (; refnum != 0; refnum--) {
 			gchar *ref;
 
-			READ_CACHE_DATA(ref, fp);
+			READ_CACHE_DATA(ref);
 			msginfo->references =
 				g_slist_prepend(msginfo->references, ref);
 		}
@@ -264,7 +297,8 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		/* if the message file doesn't exist or is changed,
 		   don't add the data */
 		if ((type == F_MH && scan_file &&
-		     folder_item_is_msg_changed(item, msginfo)) || num == 0) {
+		     folder_item_is_msg_changed(item, msginfo)) ||
+		     msginfo->msgnum == 0) {
 			procmsg_msginfo_free(msginfo);
 			item->cache_dirty = TRUE;
 		} else {
@@ -279,7 +313,13 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		}
 	}
 
-	fclose(fp);
+	g_mapped_file_free(mapfile);
+
+	if (item->cache_queue) {
+		GSList *qlist;
+		qlist = procmsg_read_cache_queue(item, scan_file);
+		mlist = g_slist_concat(mlist, qlist);
+	}
 
 	debug_print("done.\n");
 
@@ -289,6 +329,52 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 #undef READ_CACHE_DATA
 #undef READ_CACHE_DATA_INT
 
+static GSList *procmsg_read_cache_queue(FolderItem *item, gboolean scan_file)
+{
+	FolderType type;
+	MsgInfo *msginfo;
+	MsgFlags default_flags;
+	GSList *cur;
+	GSList *qlist = NULL;
+
+	g_return_val_if_fail(item != NULL, NULL);
+	g_return_val_if_fail(item->folder != NULL, NULL);
+
+	if (!item->cache_queue)
+		return NULL;
+
+	debug_print("Reading cache queue...\n");
+
+	type = FOLDER_TYPE(item->folder);
+	default_flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	default_flags.tmp_flags = 0;
+
+	for (cur = item->cache_queue; cur != NULL; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+
+		debug_print("read cache queue: %s/%d\n",
+			    item->path, msginfo->msgnum);
+
+		MSG_SET_PERM_FLAGS(msginfo->flags, default_flags.perm_flags);
+		MSG_SET_TMP_FLAGS(msginfo->flags, default_flags.tmp_flags);
+
+		if ((type == F_MH && scan_file &&
+		     folder_item_is_msg_changed(item, msginfo))) {
+			procmsg_msginfo_free(msginfo);
+			item->cache_dirty = TRUE;
+		} else {
+			msginfo->folder = item;
+			qlist = g_slist_prepend(qlist, msginfo);
+		}
+	}
+
+	g_slist_free(item->cache_queue);
+	item->cache_queue = NULL;
+	item->cache_dirty = TRUE;
+
+	return qlist;
+}
+
 static void mark_unset_new_func(gpointer key, gpointer value, gpointer data)
 {
 	MSG_UNSET_PERM_FLAGS(*((MsgFlags *)value), MSG_NEW);
@@ -402,12 +488,12 @@ void procmsg_mark_all_read(FolderItem *item)
 
 	if (item->mark_queue) {
 		GSList *cur;
-		MsgInfo *msginfo;
+		MsgFlagInfo *flaginfo;
 
 		for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
-			msginfo = (MsgInfo *)cur->data;
+			flaginfo = (MsgFlagInfo *)cur->data;
 			MSG_UNSET_PERM_FLAGS
-				(msginfo->flags, MSG_NEW|MSG_UNREAD);
+				(flaginfo->flags, MSG_NEW|MSG_UNREAD);
 		}
 		item->mark_dirty = TRUE;
 	}
@@ -534,6 +620,9 @@ void procmsg_write_cache_list(FolderItem *item, GSList *mlist)
 		procmsg_write_cache(msginfo, fp);
 	}
 
+	if (item->cache_queue)
+		procmsg_flush_cache_queue(item, fp);
+
 	fclose(fp);
 	item->cache_dirty = FALSE;
 }
@@ -611,32 +700,163 @@ void procmsg_write_flags_for_multiple_folders(GSList *mlist)
 
 void procmsg_flush_mark_queue(FolderItem *item, FILE *fp)
 {
-	MsgInfo *flaginfo;
+	MsgFlagInfo *flaginfo;
+	MsgInfo msginfo = {0};
+	gboolean append = FALSE;
+	GSList *qlist, *cur;
 
 	g_return_if_fail(item != NULL);
-	g_return_if_fail(fp != NULL);
 
-	if (item->mark_queue)
-		debug_print("flushing mark_queue...\n");
+	if (!item->mark_queue)
+		return;
 
-	while (item->mark_queue != NULL) {
-		flaginfo = (MsgInfo *)item->mark_queue->data;
-		procmsg_write_flags(flaginfo, fp);
-		procmsg_msginfo_free(flaginfo);
-		item->mark_queue = g_slist_remove(item->mark_queue, flaginfo);
+	debug_print("flushing mark_queue: %s ...\n", item->path);
+
+	if (!fp) {
+		append =  TRUE;
+		fp = procmsg_open_mark_file(item, DATA_APPEND);
+		g_return_if_fail(fp != NULL);
 	}
+
+	qlist = g_slist_reverse(item->mark_queue);
+	item->mark_queue = NULL;
+
+	for (cur = qlist; cur != NULL; cur = cur->next) {
+		flaginfo = (MsgFlagInfo *)cur->data;
+
+		msginfo.msgnum = flaginfo->msgnum;
+		msginfo.flags = flaginfo->flags;
+		procmsg_write_flags(&msginfo, fp);
+		g_free(flaginfo);
+	}
+
+	g_slist_free(qlist);
+
+	if (append)
+		fclose(fp);
 }
 
 void procmsg_add_mark_queue(FolderItem *item, gint num, MsgFlags flags)
+{
+	MsgFlagInfo *flaginfo;
+
+	flaginfo = g_new(MsgFlagInfo, 1);
+	flaginfo->msgnum = num;
+	flaginfo->flags = flags;
+	item->mark_queue = g_slist_prepend(item->mark_queue, flaginfo);
+}
+
+void procmsg_flaginfo_list_free(GSList *flaglist)
+{
+	GSList *cur;
+	MsgFlagInfo *flaginfo;
+
+	for (cur = flaglist; cur != NULL; cur = cur->next) {
+		flaginfo = (MsgFlagInfo *)cur->data;
+		g_free(flaginfo);
+	}
+	g_slist_free(flaglist);
+}
+
+void procmsg_flush_cache_queue(FolderItem *item, FILE *fp)
+{
+	MsgInfo *msginfo;
+	gboolean append = FALSE;
+	GSList *qlist, *cur;
+
+	g_return_if_fail(item != NULL);
+
+	if (!item->cache_queue)
+		return;
+
+	debug_print("flushing cache_queue: %s ...\n", item->path);
+
+	if (!fp) {
+		append =  TRUE;
+		fp = procmsg_open_cache_file(item, DATA_APPEND);
+		g_return_if_fail(fp != NULL);
+	}
+
+	qlist = g_slist_reverse(item->cache_queue);
+	item->cache_queue = NULL;
+
+	for (cur = qlist; cur != NULL; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+
+		debug_print("flush cache queue: %s/%d\n",
+			    item->path, msginfo->msgnum);
+		procmsg_write_cache(msginfo, fp);
+		procmsg_msginfo_free(msginfo);
+	}
+
+	g_slist_free(qlist);
+
+	if (append)
+		fclose(fp);
+}
+
+void procmsg_add_cache_queue(FolderItem *item, gint num, MsgInfo *msginfo)
 {
 	MsgInfo *queue_msginfo;
 
-	queue_msginfo = g_new0(MsgInfo, 1);
+	g_return_if_fail(msginfo != NULL);
+
+	queue_msginfo = procmsg_msginfo_copy(msginfo);
 	queue_msginfo->msgnum = num;
-	queue_msginfo->flags = flags;
-	item->mark_queue = g_slist_append
-		(item->mark_queue, queue_msginfo);
-	return;
+	queue_msginfo->folder = item;
+	if (queue_msginfo->file_path) {
+		g_free(queue_msginfo->file_path);
+		queue_msginfo->file_path = NULL;
+	}
+
+	debug_print("procmsg_add_cache_queue: add msg cache: %s/%d\n",
+		    item->path, num);
+	item->cache_queue = g_slist_prepend(item->cache_queue, queue_msginfo);
+}
+
+gboolean procmsg_flush_folder(FolderItem *item)
+{
+	gboolean flushed = FALSE;
+	gint n_new, n_unread, n_total, n_min, n_max;
+
+	g_return_val_if_fail(item != NULL, FALSE);
+	g_return_val_if_fail(item->folder != NULL, FALSE);
+
+	if (FOLDER_TYPE(item->folder) != F_MH || item->last_num < 0) {
+		folder_item_scan(item);
+		return TRUE;
+	}
+
+	if (item->mark_queue && !item->opened)
+		flushed = TRUE;
+	procmsg_get_mark_sum(item, &n_new, &n_unread, &n_total, &n_min, &n_max,
+			     0);
+	item->unmarked_num = 0;
+	item->new = n_new;
+	item->unread = n_unread;
+	item->total = n_total;
+
+	if (item->cache_queue && !item->opened) {
+		procmsg_flush_cache_queue(item, NULL);
+		flushed = TRUE;
+	}
+
+	if (flushed)
+		debug_print("procmsg_flush_folder: flushed %s\n", item->path);
+
+	return flushed;
+}
+
+static void procmsg_flush_folder_foreach_func(gpointer key, gpointer val,
+					      gpointer data)
+{
+	procmsg_flush_folder(FOLDER_ITEM(key));
+}
+
+void procmsg_flush_folder_foreach(GHashTable *folder_table)
+{
+	g_hash_table_foreach(folder_table, procmsg_flush_folder_foreach_func,
+			     NULL);
 }
 
 void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
@@ -751,24 +971,24 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 	}
 
 	for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
-		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		MsgFlagInfo *flaginfo = (MsgFlagInfo *)cur->data;
 
 		flags = g_hash_table_lookup(mark_table,
-					    GUINT_TO_POINTER(msginfo->msgnum));
+					    GUINT_TO_POINTER(flaginfo->msgnum));
 		if (flags != NULL)
 			g_free(flags);
 
 		flags = g_new0(MsgFlags, 1);
-		flags->perm_flags = msginfo->flags.perm_flags;
+		flags->perm_flags = flaginfo->flags.perm_flags;
 
 		g_hash_table_insert(mark_table,
-				    GUINT_TO_POINTER(msginfo->msgnum), flags);
+				    GUINT_TO_POINTER(flaginfo->msgnum), flags);
 				    
 	}
 
 	if (item->mark_queue && !item->opened) {
 		procmsg_write_mark_file(item, mark_table);
-		procmsg_msg_list_free(item->mark_queue);
+		procmsg_flaginfo_list_free(item->mark_queue);
 		item->mark_queue = NULL;
 		item->mark_dirty = FALSE;
 	}
@@ -789,7 +1009,10 @@ static void procmsg_write_mark_file(FolderItem *item, GHashTable *mark_table)
 {
 	FILE *fp;
 
-	fp = procmsg_open_mark_file(item, DATA_WRITE);
+	if ((fp = procmsg_open_mark_file(item, DATA_WRITE)) == NULL) {
+		g_warning("procmsg_write_mark_file: cannot open mark file.");
+		return;
+	}
 	g_hash_table_foreach(mark_table, write_mark_func, fp);
 	fclose(fp);
 }
@@ -804,8 +1027,16 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 
 	if (mode == DATA_WRITE) {
 		if ((fp = g_fopen(file, "wb")) == NULL) {
-			FILE_OP_ERROR(file, "fopen");
-			return NULL;
+			if (errno == EACCES) {
+				change_file_mode_rw(NULL, file);
+				if ((fp = g_fopen(file, "wb")) == NULL) {
+					FILE_OP_ERROR(file, "procmsg_open_data_file: fopen");
+					return NULL;
+				}
+			} else {
+				FILE_OP_ERROR(file, "procmsg_open_data_file: fopen");
+				return NULL;
+			}
 		}
 		if (change_file_mode_rw(fp, file) < 0)
 			FILE_OP_ERROR(file, "chmod");
@@ -815,13 +1046,25 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 	}
 
 	/* check version */
-	if ((fp = g_fopen(file, "rb")) == NULL)
-		debug_print("Mark/Cache file '%s' not found\n", file);
-	else {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
+		if (errno == EACCES) {
+			change_file_mode_rw(NULL, file);
+			if ((fp = g_fopen(file, "rb")) == NULL) {
+				FILE_OP_ERROR(file, "procmsg_open_data_file: fopen");
+			}
+		} else {
+			debug_print("Mark/Cache file '%s' not found\n", file);
+		}
+	}
+
+	if (fp) {
 		if (buf && buf_size > 0)
 			setvbuf(fp, buf, _IOFBF, buf_size);
-		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1 ||
-		    version != data_ver) {
+		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1) {
+			g_warning("%s: cannot read mark/cache file (truncated?)\n", file);
+			fclose(fp);
+			fp = NULL;
+		} else if (version != data_ver) {
 			g_message("%s: Mark/Cache version is different (%u != %u). Discarding it.\n",
 				  file, data_ver, version);
 			fclose(fp);
@@ -835,8 +1078,16 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 	if (fp) {
 		/* reopen with append mode */
 		fclose(fp);
-		if ((fp = g_fopen(file, "ab")) == NULL)
-			FILE_OP_ERROR(file, "fopen");
+		if ((fp = g_fopen(file, "ab")) == NULL) {
+			if (errno == EACCES) {
+				change_file_mode_rw(NULL, file);
+				if ((fp = g_fopen(file, "ab")) == NULL) {
+					FILE_OP_ERROR(file, "procmsg_open_data_file: fopen");
+				}
+			} else {
+				FILE_OP_ERROR(file, "procmsg_open_data_file: fopen");
+			}
+		}
 	} else {
 		/* open with overwrite mode if mark file doesn't exist or
 		   version is different */
@@ -847,19 +1098,54 @@ FILE *procmsg_open_data_file(const gchar *file, guint version,
 	return fp;
 }
 
-static FILE *procmsg_open_cache_file_with_buffer(FolderItem *item,
-						 DataOpenMode mode,
-						 gchar *buf, size_t buf_size)
+static GMappedFile *procmsg_open_cache_file_mmap(FolderItem *item,
+						 DataOpenMode mode)
 {
 	gchar *cachefile;
-	FILE *fp;
+	GMappedFile *map = NULL;
+	GError *error = NULL;
+	gsize size;
+	guint32 data_ver = 0;
+	gchar *p;
+
+	if (mode != DATA_READ)
+		return NULL;
 
 	cachefile = folder_item_get_cache_file(item);
-	fp = procmsg_open_data_file(cachefile, CACHE_VERSION, mode, buf,
-				    buf_size);
-	g_free(cachefile);
+	if (cachefile) {
+		map = g_mapped_file_new(cachefile, FALSE, &error);
+		if (!map) {
+			if (error && error->code == G_FILE_ERROR_NOENT)
+				debug_print("%s: mark/cache file not found\n", cachefile);
+			else if (error)
+				g_warning("%s: cannot open mark/cache file: %s", cachefile, error->message);
+			else
+				g_warning("%s: cannot open mark/cache file", cachefile);
+			if (error)
+				g_error_free(error);
+			g_free(cachefile);
+			return NULL;
+		}
+		size = g_mapped_file_get_length(map);
+		if (size < sizeof(data_ver)) {
+			g_warning("%s: cannot read mark/cache file (truncated?)", cachefile);
+			g_mapped_file_free(map);
+			g_free(cachefile);
+			return NULL;
+		}
+		p = g_mapped_file_get_contents(map);
+		data_ver = *(guint32 *)p;
+		if (CACHE_VERSION != data_ver) {
+			g_message("%s: Mark/Cache version is different (%u != %u). Discarding it.\n",
+				  cachefile, data_ver, CACHE_VERSION);
+			g_mapped_file_free(map);
+			g_free(cachefile);
+			return NULL;
+		}
+		g_free(cachefile);
+	}
 
-	return fp;
+	return map;
 }
 
 FILE *procmsg_open_cache_file(FolderItem *item, DataOpenMode mode)
@@ -1073,6 +1359,79 @@ gint procmsg_copy_messages(GSList *mlist)
 	return val == -1 ? -1 : 0;
 }
 
+gint procmsg_add_messages_from_queue(FolderItem *dest, GSList *mlist,
+				     gboolean is_move)
+{
+	MsgInfo *msginfo;
+	GSList *cur;
+	gchar *file;
+	FILE *fp;
+	gchar buf[BUFFSIZE];
+	gchar *dest_file;
+	gboolean is_error = FALSE;
+	FolderItem *src;
+	MsgFlags flags;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(mlist != NULL, -1);
+
+	msginfo = (MsgInfo *)mlist->data;
+	if (!msginfo || !msginfo->folder || msginfo->folder->stype != F_QUEUE ||
+	    !MSG_IS_QUEUED(msginfo->flags) || dest->stype == F_QUEUE)
+		return -1;
+
+	debug_print("procmsg_add_messages_from_queue: adding messages from queue folder\n");
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+		flags = msginfo->flags;
+		if (!MSG_IS_QUEUED(flags))
+			return -1;
+		MSG_UNSET_TMP_FLAGS(flags, MSG_QUEUED);
+		src = msginfo->folder;
+		file = procmsg_get_message_file(msginfo);
+		if (!file)
+			return -1;
+		if ((fp = g_fopen(file, "rb")) == NULL) {
+			FILE_OP_ERROR(file, "folder_item_move_msgs: fopen");
+			g_free(file);
+			return -1;
+		}
+		while (fgets(buf, sizeof(buf), fp) != NULL) {
+			if (buf[0] == '\r' || buf[0] == '\n')
+				break;
+		}
+		if (ferror(fp)) {
+			fclose(fp);
+			g_free(file);
+			return -1;
+		}
+
+		dest_file = get_tmp_file();
+		debug_print("copy queued msg: %s -> %s\n", file, dest_file);
+
+		if (copy_file_part(fp, ftell(fp), G_MAXINT, dest_file) < 0) {
+			fclose(fp);
+			is_error = TRUE;
+		} else {
+			fclose(fp);
+			if (folder_item_add_msg(dest, dest_file, &flags, TRUE) < 0) {
+				g_unlink(dest_file);
+				is_error = TRUE;
+			} else if (is_move &&
+				   folder_item_remove_msg(src, msginfo) < 0)
+				is_error = TRUE;
+		}
+
+		g_free(dest_file);
+		g_free(file);
+		if (is_error)
+			return -1;
+	}
+
+	return 0;
+}
+
 gchar *procmsg_get_message_file_path(MsgInfo *msginfo)
 {
 	gchar *path, *file;
@@ -1084,9 +1443,10 @@ gchar *procmsg_get_message_file_path(MsgInfo *msginfo)
 	else if (msginfo->file_path)
 		return g_strdup(msginfo->file_path);
 	else {
+		gchar nstr[16];
 		path = folder_item_get_path(msginfo->folder);
 		file = g_strconcat(path, G_DIR_SEPARATOR_S,
-				   itos(msginfo->msgnum), NULL);
+				   utos_buf(nstr, msginfo->msgnum), NULL);
 		g_free(path);
 	}
 
@@ -1169,7 +1529,7 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 	}
 
 	if ((fp = g_fopen(file, "rb")) == NULL) {
-		FILE_OP_ERROR(file, "fopen");
+		FILE_OP_ERROR(file, "procmsg_open_message: fopen");
 		g_free(file);
 		return NULL;
 	}
@@ -1187,17 +1547,23 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 }
 
 static DecryptMessageFunc decrypt_message_func = NULL;
+static gboolean auto_decrypt = TRUE;
 
 void procmsg_set_decrypt_message_func(DecryptMessageFunc func)
 {
 	decrypt_message_func = func;
 }
 
+void procmsg_set_auto_decrypt_message(gboolean enabled)
+{
+	auto_decrypt = enabled;
+}
+
 FILE *procmsg_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
 {
 	FILE *fp;
 
-	if (decrypt_message_func)
+	if (decrypt_message_func && auto_decrypt)
 		return decrypt_message_func(msginfo, mimeinfo);
 
 	*mimeinfo = NULL;
@@ -1242,7 +1608,7 @@ void procmsg_empty_trash(FolderItem *trash)
 	if (!trash)
 		return;
 
-	g_return_if_fail(trash->stype == F_TRASH);
+	g_return_if_fail(trash->stype == F_TRASH || trash->stype == F_JUNK);
 
 	if (trash->total > 0) {
 		debug_print("Emptying messages in %s ...\n", trash->path);
@@ -1316,6 +1682,7 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file)
 		g_warning("can't save message\n");
 		return -1;
 	}
+	procmsg_flush_folder(outbox);
 
 	return 0;
 }
@@ -1380,7 +1747,7 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 				print_id++);
 
 	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
-		FILE_OP_ERROR(prtmp, "fopen");
+		FILE_OP_ERROR(prtmp, "procmsg_print_message: fopen");
 		g_free(prtmp);
 		fclose(tmpfp);
 		return;
@@ -1473,7 +1840,7 @@ void procmsg_print_message_part(MsgInfo *msginfo, MimeInfo *partinfo,
 				get_mime_tmp_dir(), G_DIR_SEPARATOR,
 				print_id++);
 	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
-		FILE_OP_ERROR(prtmp, "fopen");
+		FILE_OP_ERROR(prtmp, "procmsg_print_message_part: fopen");
 		g_free(prtmp);
 		fclose(tmpfp);
 		return;
@@ -1490,6 +1857,267 @@ void procmsg_print_message_part(MsgInfo *msginfo, MimeInfo *partinfo,
 	g_free(prtmp);
 }
 
+/**
+ * procmsg_concat_partial_messages:
+ * @mlist: list of MsgInfo* including message/partial messages.
+ * @file: output file name of concatenated message.
+ *
+ * Concatenate @mlist which consists of message/partial messages and
+ * output to @file. If @mlist has different partial id, the first one
+ * is used.
+ *
+ * Return value: 0 on success, or -1 if failed.
+ **/
+gint procmsg_concat_partial_messages(GSList *mlist, const gchar *file)
+{
+	static HeaderEntry hentry[] = {{"Content-Type:", NULL, FALSE},
+				       {NULL, NULL, FALSE}};
+	FILE *fp;
+	gchar buf[BUFFSIZE];
+	FILE *tmp_fp;
+	gchar *part_id = NULL;
+	gint total = 0;
+	MsgInfo *msg_array[100] = {NULL};
+	MsgInfo *msginfo;
+	MimeInfo *mimeinfo;
+	GSList *cur;
+	gint i;
+
+	g_return_val_if_fail(mlist != NULL, -1);
+	g_return_val_if_fail(file != NULL, -1);
+
+	debug_print("procmsg_concat_partial_messages\n");
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		gint n = 0;
+		gint t = 0;
+		gchar *cur_id = NULL;
+
+		msginfo = (MsgInfo *)cur->data;
+
+		fp = procmsg_open_message_decrypted(msginfo, &mimeinfo);
+		if (!fp)
+			continue;
+		if (!mimeinfo->content_type ||
+		    g_ascii_strcasecmp(mimeinfo->content_type, "message/partial") != 0)
+			goto skip;
+
+		rewind(fp);
+		if (procheader_get_one_field(buf, sizeof(buf), fp, hentry) == -1)
+			goto skip;
+
+		procmime_scan_content_type_partial(buf + strlen(hentry[0].name),
+						   &t, &cur_id, &n);
+		if (n == 0 || n > 100 || t > 100 || (t > 0 && n > t)) {
+			debug_print("bad partial number (%d/%d), skip\n", n, t);
+			g_free(cur_id);
+			goto skip;
+		}
+
+		debug_print("partial: %d/%d id=%s\n", n, t, cur_id);
+		if (!part_id)
+			part_id = g_strdup(cur_id);
+		if (total == 0)
+			total = t;
+
+		if ((t > 0 && total != t) || (total > 0 && n > total) ||
+		    strcmp(part_id, cur_id) != 0) {
+			debug_print("skip\n");
+			g_free(cur_id);
+			goto skip;
+		}
+
+		msg_array[n - 1] = msginfo;
+
+		g_free(cur_id);
+skip:
+		procmime_mimeinfo_free_all(mimeinfo);
+		fclose(fp);
+	}
+
+	if (!part_id) {
+		debug_print("piece not found\n");
+		return -1;
+	}
+
+	debug_print("part_id = %s , total = %d\n", part_id, total);
+	g_free(part_id);
+
+	if (total == 0) {
+		debug_print("total number not found\n");
+		return -1;
+	}
+
+	/* check if all pieces exist */
+	for (i = 0; i < total; i++) {
+		if (msg_array[i] == NULL) {
+			debug_print("message part %d not exist\n", i + 1);
+			return -1;
+		}
+	}
+
+	/* concatenate parts */
+	if ((tmp_fp = g_fopen(file, "wb")) == NULL) {
+		FILE_OP_ERROR(file, "fopen");
+		return -1;
+	}
+
+	for (i = 0; i < total; i++) {
+		msginfo = msg_array[i];
+		off_t out_size;
+		gint empty_line_size = 0;
+
+		fp = procmsg_open_message_decrypted(msginfo, &mimeinfo);
+		if (!fp) {
+			g_warning("cannot open message part %d\n", i + 1);
+			fclose(tmp_fp);
+			g_unlink(file);
+			return -1;
+		}
+
+		/* write out first headers */
+		if (i == 0) {
+			rewind(fp);
+			while (procheader_get_one_field(buf, sizeof(buf), fp, NULL) != -1) {
+				if (!g_ascii_strncasecmp(buf, "Content-", 8) ||
+				    !g_ascii_strncasecmp(buf, "Subject", 7) ||
+				    !g_ascii_strncasecmp(buf, "Message-ID", 10) ||
+				    !g_ascii_strncasecmp(buf, "Encrypted", 9) ||
+				    !g_ascii_strncasecmp(buf, "MIME-Version", 12))
+					continue;
+				fputs(buf, tmp_fp);
+				fputs("\n", tmp_fp);
+			}
+
+			while (procheader_get_one_field(buf, sizeof(buf), fp, NULL) != -1) {
+				if (!g_ascii_strncasecmp(buf, "Content-", 8) ||
+				    !g_ascii_strncasecmp(buf, "Subject", 7) ||
+				    !g_ascii_strncasecmp(buf, "Message-ID", 10) ||
+				    !g_ascii_strncasecmp(buf, "Encrypted", 9) ||
+				    !g_ascii_strncasecmp(buf, "MIME-Version", 12)) {
+					fputs(buf, tmp_fp);
+					fputs("\n", tmp_fp);
+				}
+			}
+
+			/* header-body separator */
+			fputs("\n", tmp_fp);
+		}
+
+		out_size = get_left_file_size(fp);
+		if (out_size < 0) {
+			g_warning("cannot tell left file size of part %d\n", i + 1);
+			procmime_mimeinfo_free_all(mimeinfo);
+			fclose(fp);
+			fclose(tmp_fp);
+			g_unlink(file);
+			return -1;
+		}
+		empty_line_size = get_last_empty_line_size(fp, out_size);
+		if (empty_line_size < 0) {
+			g_warning("cannot get last empty line size of part %d\n", i + 1);
+			procmime_mimeinfo_free_all(mimeinfo);
+			fclose(fp);
+			fclose(tmp_fp);
+			g_unlink(file);
+			return -1;
+		}
+
+		if (append_file_part(fp, ftell(fp), out_size - empty_line_size,
+				     tmp_fp) < 0) {
+			g_warning("write failed\n");
+			procmime_mimeinfo_free_all(mimeinfo);
+			fclose(fp);
+			fclose(tmp_fp);
+			g_unlink(file);
+			return -1;
+		}
+
+		procmime_mimeinfo_free_all(mimeinfo);
+		fclose(fp);
+	}
+
+	fclose(tmp_fp);
+
+	return 0;
+}
+
+static gboolean procmsg_get_flags(FolderItem *item, gint num,
+				  MsgPermFlags *flags)
+{
+	FILE *fp;
+	guint32 idata;
+	gint read_num;
+	MsgPermFlags perm_flags;
+	gboolean found = FALSE;
+	GSList *cur;
+
+	if ((fp = procmsg_open_mark_file(item, DATA_READ)) == NULL)
+		return FALSE;
+
+	while (fread(&idata, sizeof(idata), 1, fp) == 1) {
+		read_num = idata;
+		if (fread(&idata, sizeof(idata), 1, fp) != 1)
+			break;
+		perm_flags = idata;
+		if (read_num == num) {
+			*flags = perm_flags;
+			found = TRUE;
+			break;
+		}
+	}
+
+	fclose(fp);
+	if (found)
+		return TRUE;
+
+	for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
+		MsgFlagInfo *flaginfo = (MsgFlagInfo *)cur->data;
+
+		if (flaginfo->msgnum == num) {
+			*flags = flaginfo->flags.perm_flags;
+			found = TRUE;
+			break;
+		}
+	}
+
+	return found;
+}
+
+MsgInfo *procmsg_get_msginfo(FolderItem *item, gint num)
+{
+	MsgInfo *msginfo;
+	FolderType type;
+
+	g_return_val_if_fail(item->folder != NULL, NULL);
+
+	msginfo = folder_item_get_msginfo(item, num);
+	if (!msginfo)
+		return NULL;
+
+	type = FOLDER_TYPE(item->folder);
+	if (type == F_MH || type == F_IMAP) {
+		if (item->stype == F_QUEUE) {
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_QUEUED);
+		} else if (item->stype == F_DRAFT) {
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_DRAFT);
+		}
+	}
+	if (type == F_IMAP) {
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_IMAP);
+	} else if (type == F_NEWS) {
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_NEWS);
+	}
+
+	if (type == F_MH || type == F_NEWS) {
+		MsgPermFlags flags = 0;
+		if (procmsg_get_flags(item, num, &flags))
+			msginfo->flags.perm_flags = flags;
+	}
+
+	return msginfo;
+}
+
 MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 {
 	MsgInfo *newmsginfo;
