@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkradiomenuitem.h>
 
 #define MENU_VAL_ID "Sylpheed::Menu::ValueID"
@@ -62,6 +63,22 @@
 				  GINT_TO_POINTER(data));		\
 }
 
+#define MENUITEM_ADD_FROM_STOCK(menu, menuitem, label, data)		\
+{									\
+	if (label)							\
+		menuitem = gtk_image_menu_item_new_from_stock(label, NULL); \
+	else {								\
+		menuitem = gtk_menu_item_new();				\
+		gtk_widget_set_sensitive(menuitem, FALSE);		\
+	}								\
+	gtk_widget_show(menuitem);					\
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);		\
+	if (data)							\
+		g_object_set_data(G_OBJECT(menuitem),			\
+				  MENU_VAL_ID,				\
+				  GINT_TO_POINTER(data));		\
+}
+
 #define MENUITEM_ADD_RADIO(menu, menuitem, widget, label, data)		\
 {									\
 	if (label) {							\
@@ -117,6 +134,11 @@ void menu_button_position	(GtkMenu		*menu,
 				 gint			*y,
 				 gboolean		*push_in,
 				 gpointer		 user_data);
+void menu_widget_position	(GtkMenu		*menu,
+				 gint			*x,
+				 gint			*y,
+				 gboolean		*push_in,
+				 gpointer		 user_data);
 
 gint menu_find_option_menu_index(GtkOptionMenu		*optmenu,
 				 gpointer		 data,
