@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,93 +54,20 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 
 
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     SHOW_NEWSGROUPS = 258,
-     SHOW_DATE = 259,
-     SHOW_FROM = 260,
-     SHOW_FULLNAME = 261,
-     SHOW_FIRST_NAME = 262,
-     SHOW_SENDER_INITIAL = 263,
-     SHOW_SUBJECT = 264,
-     SHOW_TO = 265,
-     SHOW_MESSAGEID = 266,
-     SHOW_PERCENT = 267,
-     SHOW_CC = 268,
-     SHOW_REFERENCES = 269,
-     SHOW_MESSAGE = 270,
-     SHOW_QUOTED_MESSAGE = 271,
-     SHOW_BACKSLASH = 272,
-     SHOW_TAB = 273,
-     SHOW_QUOTED_MESSAGE_NO_SIGNATURE = 274,
-     SHOW_MESSAGE_NO_SIGNATURE = 275,
-     SHOW_EOL = 276,
-     SHOW_QUESTION_MARK = 277,
-     SHOW_OPARENT = 278,
-     SHOW_CPARENT = 279,
-     QUERY_DATE = 280,
-     QUERY_FROM = 281,
-     QUERY_FULLNAME = 282,
-     QUERY_SUBJECT = 283,
-     QUERY_TO = 284,
-     QUERY_NEWSGROUPS = 285,
-     QUERY_MESSAGEID = 286,
-     QUERY_CC = 287,
-     QUERY_REFERENCES = 288,
-     OPARENT = 289,
-     CPARENT = 290,
-     CHARACTER = 291
-   };
-#endif
-/* Tokens.  */
-#define SHOW_NEWSGROUPS 258
-#define SHOW_DATE 259
-#define SHOW_FROM 260
-#define SHOW_FULLNAME 261
-#define SHOW_FIRST_NAME 262
-#define SHOW_SENDER_INITIAL 263
-#define SHOW_SUBJECT 264
-#define SHOW_TO 265
-#define SHOW_MESSAGEID 266
-#define SHOW_PERCENT 267
-#define SHOW_CC 268
-#define SHOW_REFERENCES 269
-#define SHOW_MESSAGE 270
-#define SHOW_QUOTED_MESSAGE 271
-#define SHOW_BACKSLASH 272
-#define SHOW_TAB 273
-#define SHOW_QUOTED_MESSAGE_NO_SIGNATURE 274
-#define SHOW_MESSAGE_NO_SIGNATURE 275
-#define SHOW_EOL 276
-#define SHOW_QUESTION_MARK 277
-#define SHOW_OPARENT 278
-#define SHOW_CPARENT 279
-#define QUERY_DATE 280
-#define QUERY_FROM 281
-#define QUERY_FULLNAME 282
-#define QUERY_SUBJECT 283
-#define QUERY_TO 284
-#define QUERY_NEWSGROUPS 285
-#define QUERY_MESSAGEID 286
-#define QUERY_CC 287
-#define QUERY_REFERENCES 288
-#define OPARENT 289
-#define CPARENT 290
-#define CHARACTER 291
-
-
-
-
 /* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
 #line 1 "quote_fmt_parse.y"
 
 
@@ -271,6 +197,9 @@ static int isseparator(int ch)
 }
 
 
+/* Line 189 of yacc.c  */
+#line 202 "quote_fmt_parse.c"
+
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -289,27 +218,113 @@ static int isseparator(int ch)
 # define YYTOKEN_TABLE 0
 #endif
 
+
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     SHOW_NEWSGROUPS = 258,
+     SHOW_DATE = 259,
+     SHOW_FROM = 260,
+     SHOW_FULLNAME = 261,
+     SHOW_FIRST_NAME = 262,
+     SHOW_SENDER_INITIAL = 263,
+     SHOW_SUBJECT = 264,
+     SHOW_TO = 265,
+     SHOW_MESSAGEID = 266,
+     SHOW_PERCENT = 267,
+     SHOW_CC = 268,
+     SHOW_REFERENCES = 269,
+     SHOW_MESSAGE = 270,
+     SHOW_QUOTED_MESSAGE = 271,
+     SHOW_BACKSLASH = 272,
+     SHOW_TAB = 273,
+     SHOW_QUOTED_MESSAGE_NO_SIGNATURE = 274,
+     SHOW_MESSAGE_NO_SIGNATURE = 275,
+     SHOW_EOL = 276,
+     SHOW_QUESTION_MARK = 277,
+     SHOW_OPARENT = 278,
+     SHOW_CPARENT = 279,
+     QUERY_DATE = 280,
+     QUERY_FROM = 281,
+     QUERY_FULLNAME = 282,
+     QUERY_SUBJECT = 283,
+     QUERY_TO = 284,
+     QUERY_NEWSGROUPS = 285,
+     QUERY_MESSAGEID = 286,
+     QUERY_CC = 287,
+     QUERY_REFERENCES = 288,
+     OPARENT = 289,
+     CPARENT = 290,
+     CHARACTER = 291
+   };
+#endif
+/* Tokens.  */
+#define SHOW_NEWSGROUPS 258
+#define SHOW_DATE 259
+#define SHOW_FROM 260
+#define SHOW_FULLNAME 261
+#define SHOW_FIRST_NAME 262
+#define SHOW_SENDER_INITIAL 263
+#define SHOW_SUBJECT 264
+#define SHOW_TO 265
+#define SHOW_MESSAGEID 266
+#define SHOW_PERCENT 267
+#define SHOW_CC 268
+#define SHOW_REFERENCES 269
+#define SHOW_MESSAGE 270
+#define SHOW_QUOTED_MESSAGE 271
+#define SHOW_BACKSLASH 272
+#define SHOW_TAB 273
+#define SHOW_QUOTED_MESSAGE_NO_SIGNATURE 274
+#define SHOW_MESSAGE_NO_SIGNATURE 275
+#define SHOW_EOL 276
+#define SHOW_QUESTION_MARK 277
+#define SHOW_OPARENT 278
+#define SHOW_CPARENT 279
+#define QUERY_DATE 280
+#define QUERY_FROM 281
+#define QUERY_FULLNAME 282
+#define QUERY_SUBJECT 283
+#define QUERY_TO 284
+#define QUERY_NEWSGROUPS 285
+#define QUERY_MESSAGEID 286
+#define QUERY_CC 287
+#define QUERY_REFERENCES 288
+#define OPARENT 289
+#define CPARENT 290
+#define CHARACTER 291
+
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 129 "quote_fmt_parse.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 129 "quote_fmt_parse.y"
+
 	char chr;
-}
-/* Line 187 of yacc.c.  */
-#line 300 "quote_fmt_parse.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 316 "quote_fmt_parse.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 313 "quote_fmt_parse.c"
+/* Line 264 of yacc.c  */
+#line 328 "quote_fmt_parse.c"
 
 #ifdef short
 # undef short
@@ -384,14 +399,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -472,9 +487,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -508,12 +523,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -634,8 +649,8 @@ static const char *const yytname[] =
   "QUERY_MESSAGEID", "QUERY_CC", "QUERY_REFERENCES", "OPARENT", "CPARENT",
   "CHARACTER", "$accept", "quote_fmt",
   "character_or_special_or_query_list", "character_or_special_or_query",
-  "character", "special", "query", "@1", "@2", "@3", "@4", "@5", "@6",
-  "@7", "@8", "@9", 0
+  "character", "special", "query", "$@1", "$@2", "$@3", "$@4", "$@5",
+  "$@6", "$@7", "$@8", "$@9", 0
 };
 #endif
 
@@ -938,17 +953,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -982,11 +1000,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1266,10 +1284,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1285,11 +1301,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1297,9 +1312,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1323,14 +1338,39 @@ yyparse ()
 #endif
 #endif
 {
-  
-  int yystate;
+
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1338,51 +1378,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1412,7 +1429,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1420,7 +1436,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1443,9 +1458,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1456,7 +1470,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1466,6 +1479,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1474,16 +1490,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1515,20 +1531,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1568,6 +1580,8 @@ yyreduce:
   switch (yyn)
     {
         case 6:
+
+/* Line 1455 of yacc.c  */
 #line 163 "quote_fmt_parse.y"
     {
 		INSERT_CHARACTER((yyvsp[(1) - (1)].chr));
@@ -1575,6 +1589,8 @@ yyreduce:
     break;
 
   case 9:
+
+/* Line 1455 of yacc.c  */
 #line 175 "quote_fmt_parse.y"
     {
 		if (msginfo->newsgroups)
@@ -1583,6 +1599,8 @@ yyreduce:
     break;
 
   case 10:
+
+/* Line 1455 of yacc.c  */
 #line 180 "quote_fmt_parse.y"
     {
 		if (msginfo->date)
@@ -1591,6 +1609,8 @@ yyreduce:
     break;
 
   case 11:
+
+/* Line 1455 of yacc.c  */
 #line 185 "quote_fmt_parse.y"
     {
 		if (msginfo->from)
@@ -1599,6 +1619,8 @@ yyreduce:
     break;
 
   case 12:
+
+/* Line 1455 of yacc.c  */
 #line 190 "quote_fmt_parse.y"
     {
 		if (msginfo->fromname)
@@ -1607,6 +1629,8 @@ yyreduce:
     break;
 
   case 13:
+
+/* Line 1455 of yacc.c  */
 #line 195 "quote_fmt_parse.y"
     {
 		if (msginfo->fromname) {
@@ -1626,6 +1650,8 @@ yyreduce:
     break;
 
   case 14:
+
+/* Line 1455 of yacc.c  */
 #line 211 "quote_fmt_parse.y"
     {
 #define MAX_SENDER_INITIAL 20
@@ -1656,6 +1682,8 @@ yyreduce:
     break;
 
   case 15:
+
+/* Line 1455 of yacc.c  */
 #line 238 "quote_fmt_parse.y"
     {
 		if (msginfo->subject)
@@ -1664,6 +1692,8 @@ yyreduce:
     break;
 
   case 16:
+
+/* Line 1455 of yacc.c  */
 #line 243 "quote_fmt_parse.y"
     {
 		if (msginfo->to)
@@ -1672,6 +1702,8 @@ yyreduce:
     break;
 
   case 17:
+
+/* Line 1455 of yacc.c  */
 #line 248 "quote_fmt_parse.y"
     {
 		if (msginfo->msgid)
@@ -1680,6 +1712,8 @@ yyreduce:
     break;
 
   case 18:
+
+/* Line 1455 of yacc.c  */
 #line 253 "quote_fmt_parse.y"
     {
 		INSERT("%");
@@ -1687,6 +1721,8 @@ yyreduce:
     break;
 
   case 19:
+
+/* Line 1455 of yacc.c  */
 #line 257 "quote_fmt_parse.y"
     {
 		if (msginfo->cc)
@@ -1695,6 +1731,8 @@ yyreduce:
     break;
 
   case 20:
+
+/* Line 1455 of yacc.c  */
 #line 262 "quote_fmt_parse.y"
     {
 		/* if (msginfo->references)
@@ -1703,6 +1741,8 @@ yyreduce:
     break;
 
   case 21:
+
+/* Line 1455 of yacc.c  */
 #line 267 "quote_fmt_parse.y"
     {
 		gchar buf[BUFFSIZE];
@@ -1726,6 +1766,8 @@ yyreduce:
     break;
 
   case 22:
+
+/* Line 1455 of yacc.c  */
 #line 287 "quote_fmt_parse.y"
     {
 		gchar buf[BUFFSIZE];
@@ -1751,6 +1793,8 @@ yyreduce:
     break;
 
   case 23:
+
+/* Line 1455 of yacc.c  */
 #line 309 "quote_fmt_parse.y"
     {
 		gchar buf[BUFFSIZE];
@@ -1776,6 +1820,8 @@ yyreduce:
     break;
 
   case 24:
+
+/* Line 1455 of yacc.c  */
 #line 331 "quote_fmt_parse.y"
     {
 		gchar buf[BUFFSIZE];
@@ -1803,6 +1849,8 @@ yyreduce:
     break;
 
   case 25:
+
+/* Line 1455 of yacc.c  */
 #line 355 "quote_fmt_parse.y"
     {
 		INSERT("\\");
@@ -1810,6 +1858,8 @@ yyreduce:
     break;
 
   case 26:
+
+/* Line 1455 of yacc.c  */
 #line 359 "quote_fmt_parse.y"
     {
 		INSERT("\t");
@@ -1817,6 +1867,8 @@ yyreduce:
     break;
 
   case 27:
+
+/* Line 1455 of yacc.c  */
 #line 363 "quote_fmt_parse.y"
     {
 		INSERT("\n");
@@ -1824,6 +1876,8 @@ yyreduce:
     break;
 
   case 28:
+
+/* Line 1455 of yacc.c  */
 #line 367 "quote_fmt_parse.y"
     {
 		INSERT("?");
@@ -1831,6 +1885,8 @@ yyreduce:
     break;
 
   case 29:
+
+/* Line 1455 of yacc.c  */
 #line 371 "quote_fmt_parse.y"
     {
 		INSERT("{");
@@ -1838,6 +1894,8 @@ yyreduce:
     break;
 
   case 30:
+
+/* Line 1455 of yacc.c  */
 #line 375 "quote_fmt_parse.y"
     {
 		INSERT("}");
@@ -1845,6 +1903,8 @@ yyreduce:
     break;
 
   case 31:
+
+/* Line 1455 of yacc.c  */
 #line 381 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->date != NULL);
@@ -1852,6 +1912,8 @@ yyreduce:
     break;
 
   case 32:
+
+/* Line 1455 of yacc.c  */
 #line 385 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1859,6 +1921,8 @@ yyreduce:
     break;
 
   case 33:
+
+/* Line 1455 of yacc.c  */
 #line 389 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->from != NULL);
@@ -1866,6 +1930,8 @@ yyreduce:
     break;
 
   case 34:
+
+/* Line 1455 of yacc.c  */
 #line 393 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1873,6 +1939,8 @@ yyreduce:
     break;
 
   case 35:
+
+/* Line 1455 of yacc.c  */
 #line 397 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->fromname != NULL);
@@ -1880,6 +1948,8 @@ yyreduce:
     break;
 
   case 36:
+
+/* Line 1455 of yacc.c  */
 #line 401 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1887,6 +1957,8 @@ yyreduce:
     break;
 
   case 37:
+
+/* Line 1455 of yacc.c  */
 #line 405 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->subject != NULL);
@@ -1894,6 +1966,8 @@ yyreduce:
     break;
 
   case 38:
+
+/* Line 1455 of yacc.c  */
 #line 409 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1901,6 +1975,8 @@ yyreduce:
     break;
 
   case 39:
+
+/* Line 1455 of yacc.c  */
 #line 413 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->to != NULL);
@@ -1908,6 +1984,8 @@ yyreduce:
     break;
 
   case 40:
+
+/* Line 1455 of yacc.c  */
 #line 417 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1915,6 +1993,8 @@ yyreduce:
     break;
 
   case 41:
+
+/* Line 1455 of yacc.c  */
 #line 421 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->newsgroups != NULL);
@@ -1922,6 +2002,8 @@ yyreduce:
     break;
 
   case 42:
+
+/* Line 1455 of yacc.c  */
 #line 425 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1929,6 +2011,8 @@ yyreduce:
     break;
 
   case 43:
+
+/* Line 1455 of yacc.c  */
 #line 429 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->msgid != NULL);
@@ -1936,6 +2020,8 @@ yyreduce:
     break;
 
   case 44:
+
+/* Line 1455 of yacc.c  */
 #line 433 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1943,6 +2029,8 @@ yyreduce:
     break;
 
   case 45:
+
+/* Line 1455 of yacc.c  */
 #line 437 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->cc != NULL);
@@ -1950,6 +2038,8 @@ yyreduce:
     break;
 
   case 46:
+
+/* Line 1455 of yacc.c  */
 #line 441 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1957,6 +2047,8 @@ yyreduce:
     break;
 
   case 47:
+
+/* Line 1455 of yacc.c  */
 #line 445 "quote_fmt_parse.y"
     {
 		/* add_visibility(msginfo->references != NULL); */
@@ -1964,6 +2056,8 @@ yyreduce:
     break;
 
   case 48:
+
+/* Line 1455 of yacc.c  */
 #line 449 "quote_fmt_parse.y"
     {
 		remove_visibility();
@@ -1971,8 +2065,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1976 "quote_fmt_parse.c"
+
+/* Line 1455 of yacc.c  */
+#line 2071 "quote_fmt_parse.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1983,7 +2078,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -2048,7 +2142,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -2065,7 +2159,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2122,9 +2216,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -2149,7 +2240,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2160,7 +2251,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
