@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include "mainwindow.h"
 #include "folderview.h"
 #include "prefs_account.h"
+#include "socks.h"
 #include "pop.h"
 #include "procheader.h"
 #include "procmsg.h"
@@ -94,6 +95,7 @@ static struct RPop3Window {
 	GtkAction *recv_action;
 	GtkAction *open_action;
 	GtkAction *delete_action;
+	GtkAction *update_action;
 	GtkAction *stop_action;
 
 	Pop3Session *session;
@@ -114,6 +116,8 @@ static const gchar *ui_def =
 	"      <menuitem name='Receive' action='ReceiveAction'/>"
 	"      <menuitem name='Open' action='OpenAction'/>"
 	"      <menuitem name='Delete' action='DeleteAction'/>"
+	"      <separator />"
+	"      <menuitem name='Update' action='UpdateAction'/>"
 	"      <menuitem name='Stop' action='StopAction'/>"
 	"      <separator />"
 	"      <menuitem name='Close' action='CloseAction'/>"
@@ -221,6 +225,7 @@ static void rpop3_close		(GtkButton	*button,
 static void rpop3_recv_cb	(void);
 static void rpop3_open_cb	(void);
 static void rpop3_delete_cb	(void);
+static void rpop3_update_cb	(void);
 static void rpop3_stop_cb	(void);
 static void rpop3_close_cb	(void);
 static void rpop3_about_cb	(void);
@@ -243,6 +248,7 @@ static GtkActionEntry action_entries[] = {
 	{"ReceiveAction", NULL, N_("_Get"), "<Control>G", NULL, rpop3_recv_cb},
 	{"OpenAction", GTK_STOCK_OPEN, NULL, NULL, NULL, rpop3_open_cb},
 	{"DeleteAction", GTK_STOCK_DELETE, NULL, "<Shift>Delete", NULL, rpop3_delete_cb},
+	{"UpdateAction", GTK_STOCK_REFRESH, NULL, NULL, NULL, rpop3_update_cb},
 	{"StopAction", GTK_STOCK_STOP, NULL, NULL, NULL, rpop3_stop_cb},
 	{"CloseAction", GTK_STOCK_CLOSE, NULL, NULL, NULL, rpop3_close_cb},
 	{"HelpAction", NULL, N_("_Help"), NULL, NULL, NULL},
@@ -272,6 +278,12 @@ gint rpop3_account(PrefsAccount *account)
 	rpop3_window.stop_load = FALSE;
 	rpop3_window.cancelled = FALSE;
 	rpop3_window.finished = FALSE;
+	if (POP3_SESSION(session)->uidl_table) {
+		hash_free_strings(POP3_SESSION(session)->uidl_table);
+		g_hash_table_destroy(POP3_SESSION(session)->uidl_table);
+		POP3_SESSION(session)->uidl_table =
+			g_hash_table_new(g_str_hash, g_str_equal);
+	}
 
 	/* override Pop3Session handlers */
 	session->recv_msg = rpop3_session_recv_msg;
@@ -298,6 +310,13 @@ gint rpop3_account(PrefsAccount *account)
 	while (!rpop3_window.finished)
 		gtk_main_iteration();
 
+	if (POP3_SESSION(session)->error_val == PS_AUTHFAIL &&
+	    account->tmp_pass) {
+		debug_print("rpop3_account: remove temporary password because of authentication failure\n");
+		g_free(account->tmp_pass);
+		account->tmp_pass = NULL;
+	}
+
 	rpop3_idle(FALSE);
 	session_destroy(session);
 	rpop3_clear_list();
@@ -454,6 +473,10 @@ static void rpop3_window_create(PrefsAccount *account)
 	g_object_set(action, "sensitive", FALSE, NULL);
 	rpop3_window.delete_action = action;
 
+	action = gtk_ui_manager_get_action(ui, "/RPop3Menu/File/Update");
+	g_object_set(action, "sensitive", FALSE, NULL);
+	rpop3_window.update_action = action;
+
 	stop_btn = gtk_button_new_from_stock(GTK_STOCK_STOP);
 	gtk_box_pack_start(GTK_BOX(hbbox), stop_btn, FALSE, FALSE, 0);
 	gtk_widget_set_sensitive(stop_btn, FALSE);
@@ -491,13 +514,21 @@ static void rpop3_window_create(PrefsAccount *account)
 static gint rpop3_start(Session *session)
 {
 	SessionState state;
+	PrefsAccount *ac;
+	SocksInfo *socks_info = NULL;
 
 	g_return_val_if_fail(session != NULL, -1);
 
 	rpop3_status_label_set(_("Connecting to %s:%d ..."),
 			       session->server, session->port);
 
-	if (session_connect(session, session->server, session->port) < 0) {
+	ac = POP3_SESSION(session)->ac_prefs;
+	if (ac->use_socks && ac->use_socks_for_recv) {
+		socks_info = socks_info_new(ac->socks_type, ac->proxy_host, ac->proxy_port, ac->use_proxy_auth ? ac->proxy_name : NULL, ac->use_proxy_auth ? ac->proxy_pass : NULL);
+	}
+
+	if (session_connect_full(session, session->server, session->port,
+				 socks_info) < 0) {
 		manage_window_focus_in(rpop3_window.window, NULL, NULL);
 		alertpanel_error(_("Can't connect to POP3 server: %s:%d"),
 				 session->server, session->port);
@@ -633,8 +664,9 @@ static void rpop3_idle(gboolean is_idle)
 		rpop3_window.session->state = POP3_IDLE;
 		if (POP3_PING_ITV < prefs_common.io_timeout_secs)
 			rpop3_window.ping_tag =
-				g_timeout_add(POP3_PING_ITV * 1000,
-					      rpop3_ping_cb, NULL);
+				g_timeout_add_full(G_PRIORITY_LOW,
+						   POP3_PING_ITV * 1000,
+						   rpop3_ping_cb, NULL, NULL);
 	}
 }
 
@@ -660,6 +692,8 @@ static gint rpop3_top_recv(Pop3Session *session, FILE *fp, guint len)
 	const gchar *subject, *from, *date;
 	gchar buf[1024];
 
+	session->msg[session->cur_msg].received = TRUE;
+
 	msginfo = procheader_parse_stream(fp, flags, FALSE);
 
 	msginfo->size = session->msg[session->cur_msg].size;
@@ -688,7 +722,8 @@ static gint rpop3_top_recv(Pop3Session *session, FILE *fp, guint len)
 			   -1);
 
 	rpop3_status_label_set(_("Retrieving message headers (%d / %d) ..."),
-			       session->cur_msg, session->count);
+			       session->count - session->cur_msg + 1,
+			       session->count);
 
 	return PS_SUCCESS;
 }
@@ -760,6 +795,11 @@ static gint rpop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
 	}
 
 	msginfo = procheader_parse_file(file, flags, FALSE);
+	if (!msginfo) {
+		session->error_val = PS_IOERR;
+		return -1;
+	}
+
 	msginfo->file_path = g_strdup(file);
 
 	msgview = messageview_create_with_new_window();
@@ -963,8 +1003,11 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
 				rpop3_status_label_set(_("Quitting..."));
 				pop3_logout_send(rpop3_window.session);
 			} else {
+				pop3_session->cur_msg = pop3_session->count;
 				gtk_widget_set_sensitive(rpop3_window.stop_btn,
 							 TRUE);
+				g_object_set(rpop3_window.update_action,
+					     "sensitive", FALSE, NULL);
 				g_object_set(rpop3_window.stop_action,
 					     "sensitive", TRUE, NULL);
 				rpop3_top_send(pop3_session);
@@ -1021,21 +1064,27 @@ static gint rpop3_session_recv_data_as_file_finished(Session *session,
 				rpop3_status_label_set(_("Quitting..."));
 				pop3_logout_send(rpop3_window.session);
 			} else if (!rpop3_window.stop_load &&
-				 (pop3_session->cur_msg < pop3_session->count)) {
-				pop3_session->cur_msg++;
+				 (pop3_session->cur_msg > 1)) {
+				pop3_session->cur_msg--;
 				rpop3_top_send(pop3_session);
 			} else {
-				rpop3_status_label_set
-					(_("Retrieved %d message headers"),
-					 pop3_session->cur_msg);
+				if (pop3_session->cur_msg > 1)
+					rpop3_status_label_set
+						(_("Retrieved %d (of %d) message headers"),
+						 pop3_session->count - pop3_session->cur_msg + 1, pop3_session->count);
+				else
+					rpop3_status_label_set
+						(_("Retrieved %d message headers"),
+						 pop3_session->count - pop3_session->cur_msg + 1);
 				gtk_widget_set_sensitive
 					(rpop3_window.recv_btn, TRUE);
 				gtk_widget_set_sensitive
 					(rpop3_window.open_btn, TRUE);
 				gtk_widget_set_sensitive
 					(rpop3_window.delete_btn, TRUE);
-				gtk_widget_set_sensitive
-					(rpop3_window.stop_btn, FALSE);
+				if (pop3_session->cur_msg == 1)
+					gtk_widget_set_sensitive(rpop3_window.stop_btn, FALSE);
+				gtk_button_set_label(GTK_BUTTON(rpop3_window.stop_btn), GTK_STOCK_REFRESH);
 				g_object_set(rpop3_window.recv_action,
 					     "sensitive", TRUE, NULL);
 				g_object_set(rpop3_window.open_action,
@@ -1044,6 +1093,10 @@ static gint rpop3_session_recv_data_as_file_finished(Session *session,
 					     "sensitive", TRUE, NULL);
 				g_object_set(rpop3_window.stop_action,
 					     "sensitive", FALSE, NULL);
+				if (pop3_session->cur_msg == 1)
+					g_object_set(rpop3_window.update_action, "sensitive", FALSE, NULL);
+				else
+					g_object_set(rpop3_window.update_action, "sensitive", TRUE, NULL);
 				rpop3_idle(TRUE);
 			}
 		} else
@@ -1206,9 +1259,40 @@ static void rpop3_delete(GtkButton *button, gpointer data)
 		g_array_free(array, TRUE);
 }
 
+static void rpop3_read_next(GtkButton *button, gpointer data)
+{
+	gint i;
+
+	if (rpop3_window.session->state != POP3_IDLE)
+		return;
+
+	for (i = rpop3_window.session->count; i > 0; i--) {
+		if (!rpop3_window.session->msg[i].received)
+			break;
+	}
+
+	if (i == 0)
+		return;
+
+	debug_print("rpop3_read_next: next: %d\n", i);
+	rpop3_window.session->cur_msg = i;
+
+	rpop3_window.stop_load = FALSE;
+	
+	gtk_button_set_label(GTK_BUTTON(rpop3_window.stop_btn), GTK_STOCK_STOP);
+	gtk_widget_set_sensitive(rpop3_window.stop_btn, TRUE);
+	g_object_set(rpop3_window.update_action, "sensitive", FALSE, NULL);
+	g_object_set(rpop3_window.stop_action, "sensitive", TRUE, NULL);
+	rpop3_idle(FALSE);
+	rpop3_top_send(rpop3_window.session);
+}
+
 static void rpop3_stop(GtkButton *button, gpointer data)
 {
-	rpop3_window.stop_load = TRUE;
+	if (rpop3_window.session->state == POP3_IDLE)
+		rpop3_read_next(NULL, NULL);
+	else
+		rpop3_window.stop_load = TRUE;
 }
 
 static void rpop3_close(GtkButton *button, gpointer data)
@@ -1219,7 +1303,7 @@ static void rpop3_close(GtkButton *button, gpointer data)
 		rpop3_status_label_set(_("Quitting..."));
 		rpop3_idle(FALSE);
 		pop3_logout_send(rpop3_window.session);
-	} else if (rpop3_window.session->state != POP3_DONE ||
+	} else if (rpop3_window.session->state != POP3_DONE &&
 		   rpop3_window.session->state != POP3_ERROR)
 		rpop3_window.cancelled = TRUE;
 }
@@ -1239,6 +1323,11 @@ static void rpop3_delete_cb(void)
 	rpop3_delete(NULL, NULL);
 }
 
+static void rpop3_update_cb(void)
+{
+	rpop3_stop(NULL, NULL);
+}
+
 static void rpop3_stop_cb(void)
 {
 	rpop3_stop(NULL, NULL);
