@@ -181,6 +181,9 @@ static void textview_show_header	(TextView	*textview,
 static void textview_insert_border	(TextView	*textview,
 					 GtkTextIter	*iter,
 					 gint		 padding);
+static void textview_insert_pad		(TextView	*textview,
+					 GtkTextIter	*iter,
+					 gint		 padding);
 
 static gboolean textview_key_pressed		(GtkWidget	*widget,
 						 GdkEventKey	*event,
@@ -246,6 +249,9 @@ static void textview_set_cursor			(TextView	*textview,
 static gboolean textview_uri_security_check	(TextView	*textview,
 						 RemoteURI	*uri);
 static void textview_uri_list_remove_all	(GSList		*uri_list);
+static void textview_uri_list_update_offsets	(TextView	*textview,
+						 gint		 start,
+						 gint		 add);
 
 
 TextView *textview_create(void)
@@ -326,12 +332,18 @@ static void textview_create_tags(TextView *textview)
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	GtkTextBuffer *buffer;
 	static PangoFontDescription *font_desc;
+	GtkTextIter iter;
+	GtkTextMark *mark;
 
 	if (!font_desc)
 		font_desc = gtkut_get_default_font_desc();
 
 	buffer = gtk_text_view_get_buffer(text);
 
+	gtk_text_buffer_get_end_iter(buffer, &iter);
+	mark = gtk_text_buffer_create_mark(buffer, "attach-file-pos",
+					   &iter, TRUE);
+
 	gtk_text_buffer_create_tag(buffer, "header",
 				   "pixels-above-lines", 1,
 				   "pixels-above-lines-set", TRUE,
@@ -500,6 +512,13 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
+		debug_print("textview_show_message: body_pos: %d\n",
+			    textview->body_pos);
+	} else {
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
+		gtk_text_buffer_move_mark(buffer, mark, &iter);
+		g_object_set_data(G_OBJECT(mark), "attach-file-count", GINT_TO_POINTER(0));
 	}
 
 #if USE_GPGME
@@ -613,8 +632,15 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
+		debug_print("textview_show_part: body_pos: %d\n",
+			    textview->body_pos);
 		if (!mimeinfo->main)
 			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+	} else {
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
+		gtk_text_buffer_move_mark(buffer, mark, &iter);
+		g_object_set_data(G_OBJECT(mark), "attach-file-count", GINT_TO_POINTER(0));
 	}
 
 	if (mimeinfo->mime_type == MIME_MULTIPART || is_rfc822_part)
@@ -859,7 +885,6 @@ static void textview_add_part_widget(TextView *textview, GtkTextIter *iter,
 	GdkColor fg2 = {0, 0x6000, 0x8000, 0xffff};
 
 	buffer = gtk_text_view_get_buffer(text);
-	gtk_text_buffer_insert(buffer, iter, "\n", 1);
 	anchor = gtk_text_buffer_create_child_anchor(buffer, iter);
 	ebox = gtk_event_box_new();
 	hbox = gtk_hbox_new(FALSE, 4);
@@ -907,6 +932,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	gint boundary_len = 0;
 	const gchar *charset;
 	GPtrArray *headers = NULL;
+	GtkTextMark *mark;
 
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(fp != NULL);
@@ -935,12 +961,17 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		g_snprintf(buf, sizeof(buf), "%s (%s)",
 			   mimeinfo->content_type,
 			   to_human_readable(mimeinfo->content_size));
+		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		textview_add_part_widget(textview, &iter, mimeinfo, buf);
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		headers = textview_scan_header(textview, fp, charset);
 		if (headers) {
 			textview_show_header(textview, headers);
 			procheader_header_array_destroy(headers);
+		} else {
+			mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
+			gtk_text_buffer_move_mark(buffer, mark, &iter);
+			g_object_set_data(G_OBJECT(mark), "attach-file-count", GINT_TO_POINTER(0));
 		}
 		return;
 	}
@@ -965,7 +996,6 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
-		textview_add_part_widget(textview, &iter, mimeinfo, buf);
 		if (mimeinfo->mime_type == MIME_IMAGE &&
 		    prefs_common.inline_image) {
 			GdkPixbuf *pixbuf;
@@ -974,6 +1004,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 			RemoteURI *uri;
 			gchar *uri_str;
 
+			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+			textview_add_part_widget(textview, &iter, mimeinfo, buf);
+
 			filename = procmime_get_tmp_file_name(mimeinfo);
 			if (procmime_get_part_fp(filename, fp, mimeinfo) < 0) {
 				g_warning("Can't get the image file.");
@@ -1017,14 +1050,36 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 			g_object_unref(pixbuf);
 			g_free(filename);
+		} else if (prefs_common.show_attached_files_first) {
+			gint count;
+			gint prev_pos, new_pos;
+
+			mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
+
+			gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
+			prev_pos = gtk_text_iter_get_offset(&iter);
+			count = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(mark), "attach-file-count"));
+			if (count == 0) {
+				textview_insert_pad(textview, &iter, 4);
+				gtk_text_buffer_move_mark(buffer, mark, &iter);
+			}
+			textview_add_part_widget(textview, &iter, mimeinfo, buf);
+			gtk_text_buffer_move_mark(buffer, mark, &iter);
+			new_pos = gtk_text_iter_get_offset(&iter);
+			textview_uri_list_update_offsets(textview, new_pos, new_pos - prev_pos);
+			g_object_set_data(G_OBJECT(mark), "attach-file-count", GINT_TO_POINTER(count + 1));
+			gtk_text_buffer_get_end_iter(buffer, &iter);
+		} else {
+			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+			textview_add_part_widget(textview, &iter, mimeinfo, buf);
 		}
 	} else {
+		/* text part */
+		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		if (!mimeinfo->main &&
 		    mimeinfo->parent &&
 		    mimeinfo->parent->children != mimeinfo)
 			textview_add_part_widget(textview, &iter, mimeinfo, buf);
-		else
-			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		textview_write_body(textview, mimeinfo, fp, charset);
 	}
 }
@@ -1668,6 +1723,7 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 	GtkTextIter iter;
 	Header *header;
 	gint i;
+	GtkTextMark *mark;
 
 	g_return_if_fail(headers != NULL);
 
@@ -1710,6 +1766,10 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 			(buffer, &iter, "\n", 1, "header", NULL);
 	}
 
+	mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
+	gtk_text_buffer_move_mark(buffer, mark, &iter);
+	g_object_set_data(G_OBJECT(mark), "attach-file-count", GINT_TO_POINTER(0));
+
 	textview_insert_border(textview, &iter, 0);
 }
 
@@ -1743,6 +1803,25 @@ static void textview_insert_border(TextView *textview, GtkTextIter *iter,
 			 G_CALLBACK(textview_size_allocate_cb), hline);
 }
 
+static void textview_insert_pad(TextView *textview, GtkTextIter *iter,
+				gint padding)
+{
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	GtkTextBuffer *buffer;
+	GtkTextChildAnchor *anchor;
+	GtkWidget *vbox;
+
+	buffer = gtk_text_view_get_buffer(text);
+
+	anchor = gtk_text_buffer_create_child_anchor(buffer, iter);
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_widget_set_size_request(vbox, -1, padding);
+	gtk_widget_show_all(vbox);
+	gtk_text_view_add_child_at_anchor(text, vbox, anchor);
+
+	gtk_text_buffer_insert(buffer, iter, "\n", 1);
+}
+
 gboolean textview_search_string(TextView *textview, const gchar *str,
 				gboolean case_sens)
 {
@@ -2134,6 +2213,8 @@ static RemoteURI *textview_get_uri(TextView *textview, GtkTextIter *start,
 		RemoteURI *uri_ = (RemoteURI *)cur->data;
 
 		if (start_pos == uri_->start && end_pos == uri_->end) {
+			debug_print("uri found: (%d, %d): %s\n",
+				    start_pos, end_pos, uri_->uri);
 			uri = uri_;
 			break;
 		}
@@ -2188,7 +2269,8 @@ static gboolean textview_event_after(GtkWidget *widget, GdkEvent *event,
 			ac = account_find_from_item(msginfo->folder);
 		if (ac && ac->protocol == A_NNTP)
 			ac = NULL;
-		compose_new(ac, msginfo->folder, uri->uri + 7, NULL);
+		compose_new(ac, msginfo ? msginfo->folder : NULL,
+			    uri->uri + 7, NULL);
 	} else if (uri->uri[0] == '#') {
 		/* don't open in-page link */
 	} else if (textview_uri_security_check(textview, uri) == TRUE)
@@ -2427,7 +2509,8 @@ static void textview_popup_menu_activate_open_uri_cb(GtkMenuItem *menuitem,
 			ac = account_find_from_item(msginfo->folder);
 		if (ac && ac->protocol == A_NNTP)
 			ac = NULL;
-		compose_new(ac, msginfo->folder, uri->uri + 7, NULL);
+		compose_new(ac, msginfo ? msginfo->folder : NULL,
+			    uri->uri + 7, NULL);
 	} else if (uri->uri[0] == '#') {
 		/* don't open in-page link */
 	} else if (textview_uri_security_check(textview, uri) == TRUE)
@@ -2632,3 +2715,20 @@ static void textview_uri_list_remove_all(GSList *uri_list)
 
 	g_slist_free(uri_list);
 }
+
+static void textview_uri_list_update_offsets(TextView *textview, gint start, gint add)
+{
+	GSList *cur;
+
+	debug_print("textview_uri_list_update_offsets: from %d: add %d\n", start, add);
+
+	for (cur = textview->uri_list; cur != NULL; cur = cur->next) {
+		RemoteURI *uri = (RemoteURI *)cur->data;
+
+		if (uri->start >= start) {
+			debug_print("textview_uri_list_update_offsets: update: (%d, %d) -> (%d, %d)\n", uri->start, uri->end, uri->start + add, uri->end + add);
+			uri->start += add;
+			uri->end += add;
+		}
+	}
+}
